package ch.zhaw.gartenverwaltung;

import ch.zhaw.gartenverwaltung.bootstrap.AfterInject;
import ch.zhaw.gartenverwaltung.bootstrap.AppLoader;
import ch.zhaw.gartenverwaltung.bootstrap.ChangeViewEvent;
import ch.zhaw.gartenverwaltung.bootstrap.Inject;
import ch.zhaw.gartenverwaltung.io.PlantList;
import ch.zhaw.gartenverwaltung.models.Garden;
import ch.zhaw.gartenverwaltung.io.HardinessZoneNotSetException;
import ch.zhaw.gartenverwaltung.models.PlantNotFoundException;
import ch.zhaw.gartenverwaltung.types.Crop;
import ch.zhaw.gartenverwaltung.types.Plant;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.image.ImageView;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.stage.Modality;
import javafx.stage.Stage;

import java.io.IOException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

public class MyGardenController {
    private static final Logger LOG = Logger.getLogger(MyGardenController.class.getName());
    @Inject
    AppLoader appLoader;
    @Inject
    private Garden garden;
    @Inject
    private PlantList plantList;

    @FXML
    public AnchorPane myGardenRoot;
    @FXML
    private VBox myPlants_vbox;

    @AfterInject
    @SuppressWarnings("unused")
    public void init() {
        garden.getPlantedCrops().addListener((observable, oldValue, newValue) -> {
            try {
                createPlantView(newValue);
            } catch (HardinessZoneNotSetException | IOException e) {
                LOG.log(Level.SEVERE, "Could not update view of croplist!", e);
            }
        });
        try {
            createPlantView(garden.getPlantedCrops());
        } catch (HardinessZoneNotSetException | IOException e) {
            LOG.log(Level.SEVERE, "Could not update view of croplist!", e);
        }
    }

    @FXML
    void addPlant() {
        myGardenRoot.fireEvent(new ChangeViewEvent(ChangeViewEvent.CHANGE_MAIN_VIEW, "Plants.fxml"));
    }

    private void createPlantView(List<Crop> crops) throws HardinessZoneNotSetException, IOException {
        myPlants_vbox.getChildren().clear();
        for (Crop crop : crops) {
            HBox hBox = createPlantView(crop);
            myPlants_vbox.getChildren().add(hBox);
        }
    }

    private HBox createPlantView(Crop crop) throws HardinessZoneNotSetException, IOException {
        //ToDo add better design
        Plant plant = plantList.getPlantById(Settings.getInstance().getCurrentHardinessZone(), crop.getPlantId()).get();
        HBox hBox = new HBox(10);
        ImageView imageView = new ImageView();
        imageView.setPreserveRatio(false);
        imageView.setFitHeight(100);
        imageView.setFitWidth(100);
        imageView.maxHeight(100);
        if (plant.image() != null) {
            imageView.setImage(plant.image());
        }
        hBox.setMinHeight(100);
        Label label = new Label(plant.name());
        label.setMaxWidth(2000);
        HBox.setHgrow(label, Priority.ALWAYS);
        Button details = new Button("Details");
        Button delete = new Button("delete");
        details.setOnAction(getGoToCropDetailEvent(crop));
        delete.setOnAction(getDeleteCropEvent(crop));
        hBox.getChildren().addAll(imageView, label, details, delete);
        return hBox;
    }

    private EventHandler<ActionEvent> getGoToCropDetailEvent(Crop crop) {
        return (event) -> {
            try {
                Stage stage = new Stage();
                if (appLoader.loadSceneToStage("CropDetail.fxml", stage) instanceof CropDetailController controller) {
                    controller.setPlantFromCrop(crop);
                }
                stage.initModality(Modality.APPLICATION_MODAL);
                stage.setResizable(true);
                stage.showAndWait();
            } catch (IOException | PlantNotFoundException e) {
                // TODO: show error alert
                LOG.log(Level.SEVERE, "Could not load plant details.", e);
            }
        };
    }

    private EventHandler<ActionEvent> getDeleteCropEvent(Crop crop) {
        return (event) -> {
            try {
                showConfirmation(crop);
            } catch (IOException | HardinessZoneNotSetException e) {
                e.printStackTrace();
            }
        };
    }

    private void showConfirmation(Crop crop) throws IOException, HardinessZoneNotSetException {
        Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
        alert.setTitle("Delete Crop");
        alert.setHeaderText("Are you sure want to delete this Crop?");
        alert.setContentText("Deleting this crop will remove all associated tasks from your schedule.");

        alert.showAndWait()
                .ifPresent(buttonType -> {
                    if (buttonType == ButtonType.OK) {
                        try {
                            garden.removeCrop(crop);
                        } catch (IOException e) {
                            // TODO: Show error alert
                            LOG.log(Level.SEVERE, "Could not remove crop.", e);
                        }
                    }
                });
    }
}
