package ch.zhaw.gartenverwaltung;

import ch.zhaw.gartenverwaltung.types.GrowthPhaseType;
import ch.zhaw.gartenverwaltung.types.Plant;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.util.Callback;

import java.time.LocalDate;
import java.time.MonthDay;

public class SelectSowDayController {
    private Plant selectedPlant;

    @FXML
    private DatePicker datepicker;

    @FXML
    private RadioButton harvest_radio;

    public LocalDate retrieveResult() {
        LocalDate sowDate = datepicker.getValue();
        if (harvest_radio.isSelected()) {
            //ToDo method to get current lifecycle group in plant
            sowDate = selectedPlant.sowDateFromHarvestDate(datepicker.getValue(), 0);
        }
        return sowDate;
    }

    /**
     * Set the {@link Plant} for which a date should be selected.
     *
     * @param plant Plant
     */
    public void setSelectedPlant(Plant plant) {
        selectedPlant = plant;
    }

    /**
     * add listener and set default values
     */
    @FXML
    public void initialize() {
        clearDatePickerEntries();

        Callback<DatePicker, DateCell> dayCellFactory = getDayCellFactory();
        datepicker.setDayCellFactory(dayCellFactory);
        datepicker.setEditable(false);
    }

    public void initSaveButton(Button saveButton) {
        saveButton.disableProperty().bind(datepicker.valueProperty().isNull());
    }

    /**
     * clear date picker editor when radio button is changed
     */
    private void clearDatePickerEntries() {
        harvest_radio.selectedProperty().addListener((observable, oldValue, isNowSelected) -> datepicker.setValue(null));
    }

    /**
     * date picker disable/enable dates according to selected plant: sow or harvest day
     *
     * @return cellFactory of datePicker
     */
    private Callback<DatePicker, DateCell> getDayCellFactory() {

        return (datePicker) -> new DateCell() {
            private final LocalDate today = LocalDate.now();

            @Override
            public void updateItem(LocalDate item, boolean empty) {
                super.updateItem(item, empty);
                setDisable(true);
                setStyle("-fx-background-color: #ffc0cb;");

                if (item.compareTo(today) > 0 && (!harvest_radio.isSelected() || selectedPlant.sowDateFromHarvestDate(item, 0).compareTo(today) >= 0)) {
                    GrowthPhaseType selectedPhase = harvest_radio.isSelected() ? GrowthPhaseType.HARVEST : GrowthPhaseType.SOW;
                    MonthDay minDate = selectedPlant.getMinDateForGrowthPhase(selectedPhase);
                    MonthDay maxDate = selectedPlant.getMaxDateForGrowthPhase(selectedPhase);

                    if (dateInRange(item, minDate, maxDate)) {
                        setDisable(false);
                        setStyle("-fx-background-color: #32CD32;");
                    }
                }
            }
        };
    }

    /**
     * Checks if the given {@link LocalDate} is within the given {@link MonthDay} range.
     * (regardless of year)
     *
     * @param subject The date to check
     * @param min The start of the date-range
     * @param max The end of the date-range
     * @return Whether the subject is within the range.
     */
    private boolean dateInRange(LocalDate subject, MonthDay min, MonthDay max) {
        return subject.getMonth().compareTo(min.getMonth()) >= 0 &&
                subject.getMonth().compareTo(max.getMonth()) <= 0 &&
                // if the day is less than the minimum day, the minimum month must not be equal
                (subject.getDayOfMonth() >= min.getDayOfMonth() || !subject.getMonth().equals(min.getMonth())) &&
                // if the day is greater than the maximum day, the maximum month must not be equal
                (subject.getDayOfMonth() <= max.getDayOfMonth() || !subject.getMonth().equals(max.getMonth()));
    }
}
